class Constants:
    CUSTOM_PROPS = "custom_props"

    class BASE_PROPS:
        NAME = "name"
        ID = "id"
        SUPPRESS_CONFIGURED = "suppress_configured"
        DURATION_MS = "duration_ms"
        RESULT = "result"
        EXCEPTION_TYPE = "exception_type"
        EXCEPTION_MSG = "exception_msg"
        CORRELATION_ID = "correlation_id"

        LEVEL = "log_level"
